@@ -10,9 +10,15 @@ import matplotlib.pyplot as pl
 from scipy import signal
 import time
 from metabolabpy.nmr import nmrpipeData
-from scipy.optimize import leastsq
+from scipy import optimize
 from metabolabpy.nmr import apcbc
 import metabolabpy.__init__ as ml_version
+from metabolabpy.nmr import nmrHsqc
+import sys
+try:
+    import pygamma as pg
+except:
+    pass
 
 
 class NmrData:
@@ -71,6 +77,21 @@ class NmrData:
         self.proc3 = str('')
         self.outd = str('')
         self.ver = ml_version.__version__
+        self.hsqc = nmrHsqc.NmrHsqc()
+        self.xsa = []
+        self.ysa = []
+        self.xst = []
+        self.yst = []
+        self.xss = []
+        self.yss = []
+        self.assigned_text = []
+        self.library_text = []
+        self.metabolite_text = []
+        if 'pygamma' in sys.modules:
+            self.has_pg = True
+        else:
+            self.has_pg = False
+
         # end __init__
 
     def __str__(self):  # pragma: no cover
@@ -584,6 +605,27 @@ class NmrData:
         return mat1
         # end hilbert1
 
+    def make_hsqc_spin_sys(self, c13_offset, idx=0):
+        c13_nc = self.hsqc.hsqc_data[self.hsqc.cur_metabolite].spin_systems[self.hsqc.cur_peak - 1]['c13_nc']
+        c13_idx = self.hsqc.hsqc_data[self.hsqc.cur_metabolite].spin_systems[self.hsqc.cur_peak - 1]['c13_idx']
+        chem_shift = self.hsqc.hsqc_data[self.hsqc.cur_metabolite].spin_systems[self.hsqc.cur_peak - 1]['c13_shifts']
+        if len(self.hsqc.hsqc_data[self.hsqc.cur_metabolite].c13_picked[self.hsqc.cur_peak - 1]) > 0:
+            for k in range(len(c13_idx)):
+                chem_shift[k][0] = np.mean(self.hsqc.hsqc_data[self.hsqc.cur_metabolite].c13_picked[self.hsqc.cur_peak \
+                    - 1]) + c13_offset[k] / 1000.0
+
+        j_cc = self.hsqc.hsqc_data[self.hsqc.cur_metabolite].spin_systems[self.hsqc.cur_peak - 1]['j_cc']
+        sys = pg.spin_system(sum(c13_nc[idx]))
+        sys.Omega(self.acq.sfo2)
+        for k in range(len(c13_nc[idx])):
+            sys.PPM(k, chem_shift[idx][k])
+
+        for k in range(len(j_cc[idx])):
+            sys.J(0, k + 1, j_cc[idx][k])
+
+        return sys
+        # end make_spin_sys
+
     def multiply(self, factor=1.0):
         self.spc = factor * self.spc
 
@@ -658,6 +700,31 @@ class NmrData:
         return mat
         # end phase3
 
+    def pick_local_opt(self, start=[0, 0]):
+        start_h1 = len(self.spc[0]) - self.ppm2points(start[0], 0) - 1
+        start_c13 = len(self.spc) - self.ppm2points(start[1], 1) - 1
+        check_matrix = np.array([[[1,1],[0,1],[-1,1]],[[1,0],[0,0],[-1,0]],[[1,-1],[0,-1],[-1,-1]]])
+        local_spc = np.array([[]])
+        local_spc.resize(3,3)
+        for k in range(3):
+            for l in range(3):
+                local_spc[k][l] = self.spc[start_c13 + check_matrix[k][l][0]][start_h1 + check_matrix[k][l][1]].real
+
+        max_pos = np.where(local_spc == local_spc.max())
+        while check_matrix[max_pos][0][0] != 0 or check_matrix[max_pos][0][1] != 0:
+            start_h1 += check_matrix[max_pos][0][1]
+            start_c13 += check_matrix[max_pos][0][0]
+            for k in range(3):
+                for l in range(3):
+                    local_spc[k][l] = self.spc[start_c13 + check_matrix[k][l][0]][start_h1 + check_matrix[k][l][1]].real
+
+            max_pos = np.where(local_spc == local_spc.max())
+
+        start_h1 = self.points2ppm(len(self.spc[0]) - start_h1 - 1, 0)
+        start_c13 = self.points2ppm(len(self.spc) - start_c13 - 1, 1)
+        return [start_h1, start_c13]
+        # end pick_local_opt
+
     def points2hz(self, points, dim=0):
         sw = self.acq.sw_h[dim]
         if dim == 0:
@@ -705,6 +772,7 @@ class NmrData:
 
             npts = int(len(self.spc))
 
+        #points = np.round(((ppm - self.ref_shift[dim]) * (sfo / sw) + self.ref_point[dim] / (npts - 1)) * (npts - 1))
         points = np.round(((ppm - self.ref_shift[dim]) * (sfo / sw) + self.ref_point[dim] / (npts - 1)) * (npts - 1))
         return points.astype(int)
         # end ppm2points
@@ -994,8 +1062,9 @@ class NmrData:
                 self.outd = fid.read()
                 fid.close()
 
-            self.acq.sfo2 = self.proc.sf[1]
-            self.display.y_label = self.proc.axis_nucleus[1]
+            self.acq.sfo2 = self.acq.bf2 + self.acq.o2/1000000.0 #self.proc.sf[1]
+            if self.proc.axis_nucleus[1] != 'off':
+                self.display.y_label = self.proc.axis_nucleus[1]
 
         else:
             title_file1 = self.data_set_name + os.sep + self.data_set_number + os.sep + 'text'
@@ -1174,6 +1243,330 @@ class NmrData:
 
         # end set_window_function
 
+    def sim_hsqc_1d(self):
+        sim_spc = np.array([], dtype=complex)
+        n_points = self.proc.n_points[1]
+        c13_nc = self.hsqc.hsqc_data[self.hsqc.cur_metabolite].spin_systems[self.hsqc.cur_peak - 1]['c13_nc']
+        perc = self.hsqc.hsqc_data[self.hsqc.cur_metabolite].spin_systems[self.hsqc.cur_peak - 1]['contribution']
+        n_spin_sys = len(self.hsqc.hsqc_data[self.hsqc.cur_metabolite].spin_systems[self.hsqc.cur_peak - 1]['c13_shifts'])
+        intensity = self.hsqc.hsqc_data[self.hsqc.cur_metabolite].intensities[self.hsqc.cur_peak - 1]
+        offset = ((self.proc.n_points[1] - self.ref_point[1])*self.acq.sw[1] / self.proc.n_points[1] + self.ref_shift[1] - self.acq.sw[1]) * self.acq.sfo2
+        sim_spc.resize(1, self.proc.n_points[1])
+        for k in range(n_spin_sys):
+            sys = self.make_hsqc_spin_sys(self.hsqc.hsqc_data[self.hsqc.cur_metabolite].spin_systems[self.hsqc.cur_peak - 1]['c13_offset'], k)
+            sys.offsetShifts(offset)
+            sim_spc[0] += intensity * self.c13spc1d(sys, perc[k]) / sum(c13_nc[k])
+
+        if intensity == 1.0:
+            spc_max = np.array([], dtype=complex)
+            spin_number = self.hsqc.cur_peak
+            hd = self.hsqc.hsqc_data[self.hsqc.cur_metabolite]
+            h1_shift = hd.h1_shifts[spin_number - 1]
+            max_idx = np.where(sim_spc[0] == np.max(sim_spc[0]))[0][0]
+            h1_picked = self.hsqc.hsqc_data[self.hsqc.cur_metabolite].h1_picked
+            if len(h1_picked[spin_number - 1]) > 0:
+                h1_pos = np.mean(h1_picked[spin_number - 1])
+            else:
+                h1_pos = h1_shift
+
+            h1_pts = len(self.spc[0]) - self.ppm2points(h1_pos, 0) - 1
+            spc_max.resize(len(sim_spc[0]))
+            for dd in range(len(sim_spc[0])):
+                spc_max[dd] = self.spc[dd][h1_pts]
+
+            intensity = spc_max[max_idx].real / sim_spc[0][max_idx].real
+            self.hsqc.hsqc_data[self.hsqc.cur_metabolite].intensities[self.hsqc.cur_peak - 1] = intensity
+
+        self.hsqc.hsqc_data[self.hsqc.cur_metabolite].sim_spc[self.hsqc.cur_peak - 1] = sim_spc[0].real
+        # end sim_hsqc_1d
+
+    def fit_hsqc_1d(self):
+        fit_parameters = []
+        c13_offset = []
+        contribution = []
+        for k in range(len(self.hsqc.hsqc_data[self.hsqc.cur_metabolite].spin_systems[self.hsqc.cur_peak - 1]['c13_offset'])):
+            c13_offset.append(self.hsqc.hsqc_data[self.hsqc.cur_metabolite].spin_systems[self.hsqc.cur_peak - 1]['c13_offset'][k])
+
+        for k in range(len(self.hsqc.hsqc_data[self.hsqc.cur_metabolite].spin_systems[self.hsqc.cur_peak - 1]['c13_offset'])):
+            contribution.append(self.hsqc.hsqc_data[self.hsqc.cur_metabolite].spin_systems[self.hsqc.cur_peak - 1]['contribution'][k])
+
+        if self.hsqc.fit_zero_percentages == False:
+            c13_offset = np.array(c13_offset)[np.where(np.array(contribution) != 0)]
+            contribution = np.array(contribution)[np.where(np.array(contribution) != 0)]
+
+        if self.hsqc.fit_chemical_shifts:
+            for k in range(len(c13_offset)):
+                fit_parameters.append(c13_offset[k])
+
+        if self.hsqc.fit_percentages:
+            for k in range(len(contribution)):
+                fit_parameters.append(contribution[k])
+
+        n_fit_parameters = len(fit_parameters)
+        eval_parameters = optimize.minimize(self.fct_hsqc_1d, fit_parameters, method='BFGS', options={'gtol': 1e-10, 'maxiter': 1e12})
+        #eval_parameters = optimize.leastsq(self.fct_hsqc_1d, fit_parameters, ftol=1e-12, xtol=1e-12, maxfev=1e10)
+        e_pars = np.array(eval_parameters.x).tolist()
+        #e_pars_array = np.array(e_pars)
+        #e_pars_array[np.where(e_pars_array < 0)] = 0
+        #e_pars = eval_parameters
+        if self.hsqc.fit_chemical_shifts and self.hsqc.fit_percentages:
+            contribution = ( np.array(e_pars[int(n_fit_parameters/2):]) * 100.0 / np.array(e_pars[int(n_fit_parameters/2):]).sum()).tolist()
+            for k in range(len(contribution)):
+                if contribution[k] < 0:
+                    contribution[k] = 0
+
+            contribution = ( np.array(contribution) * 100.0 / np.array(contribution).sum()).tolist()
+            for k in range(len(contribution)):
+                contribution[k] = round(contribution[k], 3)
+
+            c13_offset = e_pars[:int(n_fit_parameters/2)]
+            for k in range(len(c13_offset)):
+                if c13_offset[k] > 100 or c13_offset[k] < -100:
+                    c13_offset[k] = 0
+
+                if contribution[k] == 0:
+                    c13_offset[k] = 0
+
+                c13_offset[k] = round(c13_offset[k], 2)
+
+            if self.hsqc.fit_zero_percentages:
+                self.hsqc.hsqc_data[self.hsqc.cur_metabolite].spin_systems[self.hsqc.cur_peak - 1]['c13_offset'] = c13_offset
+                self.hsqc.hsqc_data[self.hsqc.cur_metabolite].spin_systems[self.hsqc.cur_peak - 1]['contribution'] = contribution
+                for k in range(len(self.hsqc.hsqc_data[self.hsqc.cur_metabolite].spin_systems[self.hsqc.cur_peak - 1]['c13_idx'])):
+                    idx = ' '.join(str(e) for e in self.hsqc.hsqc_data[self.hsqc.cur_metabolite].spin_systems[self.hsqc.cur_peak - 1]['c13_idx'][k])
+                    self.hsqc.hsqc_data[self.hsqc.cur_metabolite].c13_offset[idx] = c13_offset[k]
+
+            else:
+                c13_offset2 = self.hsqc.hsqc_data[self.hsqc.cur_metabolite].spin_systems[self.hsqc.cur_peak - 1]['c13_offset']
+                contribution2 = self.hsqc.hsqc_data[self.hsqc.cur_metabolite].spin_systems[self.hsqc.cur_peak - 1]['contribution']
+                contribution2 = np.array(contribution2)
+                c13_offset2 = np.array(c13_offset2)
+                c13_offset2[np.where(contribution2 != 0)] = np.array(c13_offset)
+                contribution2[np.where(contribution2 != 0)] = np.array(contribution)
+                self.hsqc.hsqc_data[self.hsqc.cur_metabolite].spin_systems[self.hsqc.cur_peak - 1]['c13_offset'] = c13_offset2.tolist()
+                self.hsqc.hsqc_data[self.hsqc.cur_metabolite].spin_systems[self.hsqc.cur_peak - 1]['contribution'] = contribution2.tolist()
+                for k in range(len(self.hsqc.hsqc_data[self.hsqc.cur_metabolite].spin_systems[self.hsqc.cur_peak - 1]['c13_idx'])):
+                    idx = ' '.join(str(e) for e in self.hsqc.hsqc_data[self.hsqc.cur_metabolite].spin_systems[self.hsqc.cur_peak - 1]['c13_idx'][k])
+                    self.hsqc.hsqc_data[self.hsqc.cur_metabolite].c13_offset[idx] = c13_offset2[k]
+
+
+        elif self.hsqc.fit_chemical_shifts:
+            if self.hsqc.fit_zero_percentages:
+                c13_offset = e_pars
+
+            else:
+                c13_offset = self.hsqc.hsqc_data[self.hsqc.cur_metabolite].spin_systems[self.hsqc.cur_peak - 1]['c13_offset']
+                c13_offset = np.array(c13_offset)
+                contribution = self.hsqc.hsqc_data[self.hsqc.cur_metabolite].spin_systems[self.hsqc.cur_peak - 1]['contribution']
+                contribution = np.array(contribution)
+                c13_offset[np.where(contribution != 0)] = np.array(e_pars)
+                c13_offset = c13_offset.tolist()
+
+            for k in range(len(c13_offset)):
+                if c13_offset[k] > 100 or c13_offset[k] < -100:
+                    c13_offset[k] = 0
+
+                if self.hsqc.hsqc_data[self.hsqc.cur_metabolite].spin_systems[self.hsqc.cur_peak - 1]['contribution'][k] < 0.8:
+                    c13_offset[k] = 0
+
+                c13_offset[k] = round(c13_offset[k], 2)
+
+            self.hsqc.hsqc_data[self.hsqc.cur_metabolite].spin_systems[self.hsqc.cur_peak - 1]['c13_offset'] = c13_offset
+            for k in range(len(self.hsqc.hsqc_data[self.hsqc.cur_metabolite].spin_systems[self.hsqc.cur_peak - 1]['c13_idx'])):
+                idx = ' '.join(str(e) for e in self.hsqc.hsqc_data[self.hsqc.cur_metabolite].spin_systems[self.hsqc.cur_peak - 1]['c13_idx'][k])
+                self.hsqc.hsqc_data[self.hsqc.cur_metabolite].c13_offset[idx] = c13_offset[k]
+
+        else:
+            if self.hsqc.fit_zero_percentages:
+                contribution = ( np.array(e_pars) * 100.0 / np.array(e_pars).sum()).tolist()
+            else:
+                contribution2 = self.hsqc.hsqc_data[self.hsqc.cur_metabolite].spin_systems[self.hsqc.cur_peak - 1]['contribution']
+                contribution2 = np.array(contribution2)
+                contribution2[np.where(contribution2 != 0)] = np.array(e_pars)
+                contribution = ( contribution2 * 100.0 / contribution2.sum() ).tolist()
+
+
+            for k in range(len(contribution)):
+                if contribution[k] < 0:
+                    contribution[k] = 0
+
+            contribution = ( np.array(contribution) * 100.0 / np.array(contribution).sum()).tolist()
+            for k in range(len(contribution)):
+                contribution[k] = round(contribution[k], 3)
+
+            self.hsqc.hsqc_data[self.hsqc.cur_metabolite].spin_systems[self.hsqc.cur_peak - 1]['contribution'] = contribution
+
+        # end fit_hsqc_1d
+
+    def fct_hsqc_1d(self, fit_parameters=[]):
+        n_fit_pars = len(fit_parameters)
+        if n_fit_pars == 0:
+            return []
+
+        if self.hsqc.fit_chemical_shifts and self.hsqc.fit_percentages:
+            if self.hsqc.fit_zero_percentages:
+                c13_offset = fit_parameters[:int(n_fit_pars/2)]
+                perc = fit_parameters[int(n_fit_pars/2):]
+            else:
+                perc = self.hsqc.hsqc_data[self.hsqc.cur_metabolite].spin_systems[self.hsqc.cur_peak - 1]['contribution']
+                c13_offset = self.hsqc.hsqc_data[self.hsqc.cur_metabolite].spin_systems[self.hsqc.cur_peak - 1]['c13_offset']
+                c13_offset = np.array(c13_offset)
+                c13_offset[np.where(np.array(perc) != 0)] = np.array(fit_parameters[:int(n_fit_pars/2)])
+                c13_offset = c13_offset.tolist()
+                perc = np.array(perc)
+                perc[np.where(perc != 0)] = np.array(fit_parameters[int(n_fit_pars/2):])
+                perc = perc.tolist()
+
+        elif self.hsqc.fit_chemical_shifts:
+            perc = self.hsqc.hsqc_data[self.hsqc.cur_metabolite].spin_systems[self.hsqc.cur_peak - 1]['contribution']
+            if self.hsqc.fit_zero_percentages:
+                c13_offset = fit_parameters
+            else:
+                c13_offset = self.hsqc.hsqc_data[self.hsqc.cur_metabolite].spin_systems[self.hsqc.cur_peak - 1]['c13_offset']
+                c13_offset = np.array(c13_offset)
+                c13_offset[np.where(np.array(perc) != 0)] = np.array(fit_parameters)
+                c13_offset = c13_offset.tolist()
+
+        else:
+            c13_offset = self.hsqc.hsqc_data[self.hsqc.cur_metabolite].spin_systems[self.hsqc.cur_peak - 1]['c13_offset']
+            if self.hsqc.fit_zero_percentages:
+                perc = fit_parameters
+            else:
+                perc = self.hsqc.hsqc_data[self.hsqc.cur_metabolite].spin_systems[self.hsqc.cur_peak - 1]['contribution']
+                perc = np.array(perc)
+                perc[np.where(perc != 0)] = np.array(fit_parameters)
+                perc = perc.tolist()
+
+        sim_spc = np.array([], dtype=complex)
+        n_points = self.proc.n_points[1]
+        c13_nc = self.hsqc.hsqc_data[self.hsqc.cur_metabolite].spin_systems[self.hsqc.cur_peak - 1]['c13_nc']
+        n_spin_sys = len(self.hsqc.hsqc_data[self.hsqc.cur_metabolite].spin_systems[self.hsqc.cur_peak - 1]['c13_shifts'])
+        intensity = self.hsqc.hsqc_data[self.hsqc.cur_metabolite].intensities[self.hsqc.cur_peak - 1]
+        offset = ((self.proc.n_points[1] - self.ref_point[1])*self.acq.sw[1] / self.proc.n_points[1] + self.ref_shift[1] - self.acq.sw[1]) * self.acq.sfo2
+        sim_spc.resize(1, self.proc.n_points[1])
+        for k in range(n_spin_sys):
+            sys = self.make_hsqc_spin_sys(c13_offset, k)
+            sys.offsetShifts(offset)
+            sim_spc[0] += intensity * self.c13spc1d(sys, perc[k]) / sum(c13_nc[k])
+
+        spc_max = np.array([], dtype=complex)
+        spin_number = self.hsqc.cur_peak
+        hd = self.hsqc.hsqc_data[self.hsqc.cur_metabolite]
+        max_idx = np.where(sim_spc[0] == np.max(sim_spc[0]))[0][0]
+        h1_shift = hd.h1_shifts[spin_number - 1]
+        h1_shift = self.hsqc.hsqc_data[self.hsqc.cur_metabolite].h1_shifts[self.hsqc.cur_peak -1]
+        c13_shift = self.hsqc.hsqc_data[self.hsqc.cur_metabolite].c13_shifts[hd.h1_index[self.hsqc.cur_peak - 1] - 1]
+        h1_picked = self.hsqc.hsqc_data[self.hsqc.cur_metabolite].h1_picked[self.hsqc.cur_peak -1]
+        c13_picked = self.hsqc.hsqc_data[self.hsqc.cur_metabolite].c13_picked[self.hsqc.cur_peak - 1]
+        if len(c13_picked) > 0:
+            h1_centre = np.mean(h1_picked)
+            c13_centre = np.mean(c13_picked)
+        else:
+            h1_centre = h1_shift
+            c13_centre = c13_shift
+
+        if self.hsqc.autoscale_j == True:
+            scale = self.hsqc.j_scale
+        else:
+            scale = 1.0
+
+        c13_beg = c13_centre + self.hsqc.range_c * scale
+        c13_end = c13_centre - self.hsqc.range_c * scale
+        h1_picked = self.hsqc.hsqc_data[self.hsqc.cur_metabolite].h1_picked
+        if len(h1_picked[spin_number - 1]) > 0:
+            h1_pos = np.mean(h1_picked[spin_number - 1])
+        else:
+            h1_pos = h1_shift
+
+        h1_pts = len(self.spc[0]) - self.ppm2points(h1_pos, 0) - 1
+        spc_max.resize(len(sim_spc[0]))
+        for dd in range(len(sim_spc[0])):
+            spc_max[dd] = self.spc[dd][h1_pts]
+
+        intensity = spc_max[max_idx].real / sim_spc[0][max_idx].real
+        #self.hsqc.hsqc_data[self.hsqc.cur_metabolite].intensities[self.hsqc.cur_peak - 1] = intensity
+        #self.hsqc.hsqc_data[self.hsqc.cur_metabolite].sim_spc[self.hsqc.cur_peak - 1] = sim_spc[0].real
+        c13_pts = len(self.spc)
+        c13_pts1 = c13_pts - self.ppm2points(c13_beg, 1) - 1
+        c13_pts2 = c13_pts - self.ppm2points(c13_end, 1) - 1
+        c13_range = range(c13_pts1, c13_pts2)
+        #print("h1_pts: {}, c13_range: {}".format(h1_pts, c13_range))
+        h1_picked = self.hsqc.hsqc_data[self.hsqc.cur_metabolite].h1_picked[self.hsqc.cur_peak - 1]
+        h1_pos = np.mean(h1_picked)
+        h1_pts = len(self.spc[0]) - self.ppm2points(h1_pos, 0) - 1
+        err = []
+        for k in c13_range:
+            err.append((sim_spc[0][k].real * intensity - self.spc[k][h1_pts.real]) ** 2)
+            #err.append((sim_spc[0][k].real/sim_spc[0][k].real.max() - self.spc[k][h1_pts].real/self.spc[k][h1_pts].real.max()) ** 2)
+
+        return np.array(err).sum()
+        # end fct_hsqc_1d
+
+    def c13spc1d(self, sys, perc=100.0, angle=90.0, dly_n=0.0):
+        jres = self.acq.cnst[18]
+        jres_n = 1.0
+        for k in range(0, sys.spins() - 1):
+            for l in range(k + 1, sys.spins()):
+                if (sys.J(k, l) > 10000.0):
+                    sys.J(k, l, sys.J(k, l) * jres_n / 100000.0)
+                    sys.isotope(l, '15N')
+                else:
+                    sys.J(k, l, sys.J(k, l) * jres)
+
+        sw = self.acq.sw_h[1]
+        td = int(self.acq.n_data_points[1] * 2)
+        npts = self.proc.n_points[1]
+        r2 = self.hsqc.hsqc_data[self.hsqc.cur_metabolite].r2[self.hsqc.cur_peak - 1]
+        echo_time = self.hsqc.echo_time / 1000.0
+        dt = 1.0 / sw
+        sigma0 = pg.sigma_eq(sys)
+        H = pg.Hcs(sys) + pg.HJw(sys)
+        D = pg.Fm(sys, "1H")
+        fid = pg.row_vector(td)
+        sigma1 = pg.Iypuls(sys, sigma0, 0, 90.0)
+        sigma1 = pg.evolve(pg.gen_op(sigma1), pg.gen_op(H), dly_n / jres_n)
+        sigma1 = pg.Ixpuls(sys, sigma1, "1H", 180.0)
+        sigma1 = pg.Ixpuls(sys, sigma1, "15N", 180.0)
+        sigma1 = pg.evolve(pg.gen_op(sigma1), pg.gen_op(H), dly_n / jres_n)
+        sigma1 = pg.evolve(pg.gen_op(sigma1), pg.gen_op(H), echo_time / jres)
+        sigma1 = pg.Ixpuls(sys, sigma1, "1H", 180.0)
+        sigma1 = pg.evolve(pg.gen_op(sigma1), pg.gen_op(H), echo_time / jres)
+        fid = fid + pg.FID(pg.gen_op(sigma1), pg.gen_op(D), pg.gen_op(H), dt, td)
+        spc1 = np.array([], dtype=complex)
+        spc1.resize(1, td)
+        spc = np.array([], dtype=complex)
+        spc.resize(1, npts)
+        spc2 = np.array([], dtype=complex)
+        spc2.resize(1, npts)
+        for k in range(td):
+            spc1[0][k] = fid.getRe(k) - 1j*fid.getIm(k)
+            spc1[0][k] *= np.exp(-r2 * dt * k)
+
+        #print("maxFID: {}".format(np.max(spc1[0])))
+        spc1 = self.wdwf_qsin(spc1, angle)
+        for k in range(td):
+            spc[0][k] = spc1[0][k]
+
+        spc = np.fft.fft(spc[0])
+        #print("mmax: {}!".format(np.max(spc3.real)))
+        for k in range(npts):
+            spc2[0][k] = spc[k] - spc[0]
+
+        spc2[0] = perc * spc2[0].real / 100.0
+        #print("len(spc2): {}, len(spc2[0]): {}".format(len(spc2), len(spc2[0])))
+        return spc2[0].real
+        # end c13spc1d
+
+    def wdwf_qsin(self, fid, angle=90.0):
+        if (angle > -1.0):
+            npts = len(fid[0])
+            t = np.linspace(angle * np.pi / 180.0, np.pi, npts)
+            for k in range(npts):
+                fid[0][k] = fid[0][k].real * np.sin(t[k]) * np.sin(t[k]) + 1j * fid[0][k].imag * np.sin(t[k]) * np.sin(
+                    t[k])
+
+        return fid
+
     def smo(self, fid):
         x = np.linspace(0, len(fid) - 1, len(fid))
         f0 = np.copy(fid)
