@@ -44,6 +44,9 @@ class NmrDataSet:
         self.peak_fill = False
         self.file_version = 0.0
         self.wb = []
+        self.old_data_set = -1
+        self.old_data_exp = -1
+        self.hsqc_spin_sys_connected = True
         # end __init__
 
     def add_peak(self, start_end=np.array([], dtype='float64'), peak_label=''):
@@ -806,7 +809,7 @@ class NmrDataSet:
                 nd2 = nd.NmrData()
                 # nd2.file_version = n.nmrdat[0][0].ver
                 for kk in nd2.__dict__.keys():
-                    if kk != 'acq' and kk != 'proc' and kk != 'display' and kk != 'apc':
+                    if kk != 'acq' and kk != 'proc' and kk != 'display' and kk != 'apc' and kk != 'hsqc':
                         kk2 = kk
                         str_idx = kk2.find('_')
                         while str_idx != -1:
@@ -908,6 +911,26 @@ class NmrDataSet:
 
                             nd2.apc = ac
 
+                    elif kk == 'hsqc':
+                        if hasattr(n, kk):
+                            h = n.hsqc
+                            hp = nd2.hsqc
+                            for kkk in hp.__dict__.keys():
+                                if hasattr(h, kkk):
+                                    #if kkk != 'hsqc_data':
+                                    exec('hp.' + kkk + '=h.' + kkk)
+                                    #
+                                    #else:
+                                    #    hd = h.hsqc_data
+                                    #    hdp = hp.hsqc_data
+                                    #    for kkkk in hdp.__dict__.keys():
+                                    #        if hasattr(hd, kkkk):
+                                    #            exec('hdp.' + kkkk + '=hd.' + kkkk)
+                                    #
+                                    #    hp.hsqc_data = hdp
+
+                            nd2.hsqc = hp
+
                 if vver == '0.1':
                     nd2.ver = '0.1'
 
@@ -1111,7 +1134,11 @@ class NmrDataSet:
         self.pp.init(len(self.nmrdat[self.s]))
         # end pre_proc_init
 
-    def read_spc(self, data_set_name, data_set_number):
+    def read_spc(self, data_set_name, data_set_number, dataset=1):
+        if len(self.nmrdat) < dataset:
+            self.nmrdat.append([])
+
+        self.s = dataset - 1
         self.e = len(self.nmrdat[self.s])
         nd1 = nd.NmrData()
         nd1.data_set_name = data_set_name
@@ -1120,14 +1147,14 @@ class NmrDataSet:
         self.nmrdat[self.s].append(nd1)
         # end read_spc
 
-    def read_spcs(self, data_path, data_exp):
+    def read_spcs(self, data_path, data_exp, dataset=1):
         if len(data_exp) > 1:
             for k in range(len(data_exp)):
-                self.read_spc(data_path[0], str(data_exp[k]))
+                self.read_spc(data_path[0], str(data_exp[k]), dataset)
 
         else:
             for k in range(len(data_path)):
-                self.read_spc(data_path[k], str(data_exp[0]))
+                self.read_spc(data_path[k], str(data_exp[0]), dataset)
 
     # end read_spcs
 
@@ -1138,6 +1165,9 @@ class NmrDataSet:
         nd1.data_set_number = data_set_number
         nd1.read_spc()
         nd1.read_pipe_2d(data_set_name + os.sep + data_set_number + '.proc', proc_data_name)
+        nd1.acq.sw[0] = nd1.acq.sw[0]*len(nd1.spc[0])/2**math.ceil(math.log(len(nd1.spc[0]),2))
+        nd1.acq.sw_h[0] = nd1.acq.sw_h[0]*len(nd1.spc[0])/2**math.ceil(math.log(len(nd1.spc[0]),2))
+        nd1.calc_ppm()
         self.nmrdat[self.s].append(nd1)
         # end read_spc
 
